git/list[1] front-page[2] threads[3] people[4] search[5] about
wed 2026-10-07 17:27 UTC

[PATCH v2 05/17] odb/source: introduce source type for robustness

From
Patrick Steinhardt <ps@pks.im>
Date
Mar 5, 2026, 14:19 UTC
Message-ID
<20260305-b4-pks-odb-source-pluggable-v2-5-3290bfd1f444@pks.im>
In-Reply-To
<20260305-b4-pks-odb-source-pluggable-v2-0-3290bfd1f444@pks.im>

When a caller holds a `struct odb_source`, they have no way of telling what type the source is. This doesn't really cause any problems in the current status quo as we only have a single type anyway, "files". But going forward we expect to add more types, and if so it will become necessary to tell the sources apart.

Introduce a new enum to cover this use case and assert that the given source actually matches the target source when performing the downcast.

Signed-off-by: Patrick Steinhardt <ps@pks.im>
---
 odb/source-files.c |  2 +-
 odb/source-files.h |  5 ++++-
 odb/source.c       |  2 ++
 odb/source.h       | 15 +++++++++++++++
 4 files changed, 22 insertions(+), 2 deletions(-)
diff --git a/odb/source-files.c b/odb/source-files.c
index df0ea9ee62..7496e1d9f8 100644
--- a/odb/source-files.c
+++ b/odb/source-files.c
@@ -36,7 +36,7 @@ struct odb_source_files *odb_source_files_new(struct object_database *odb,
 	struct odb_source_files *files;
 
 	CALLOC_ARRAY(files, 1);
-	odb_source_init(&files->base, odb, path, local);
+	odb_source_init(&files->base, odb, ODB_SOURCE_FILES, path, local);
 	files->loose = odb_source_loose_new(&files->base);
 	files->packed = packfile_store_new(&files->base);
 
diff --git a/odb/source.c b/odb/source.c
index d8b2176a94..c7dcc528f6 100644
--- a/odb/source.c
+++ b/odb/source.c
@@ -13,10 +13,12 @@ struct odb_source *odb_source_new(struct object_database *odb,
 
 void odb_source_init(struct odb_source *source,
 		     struct object_database *odb,
+		     enum odb_source_type type,
 		     const char *path,
 		     bool local)
 {
 	source->odb = odb;
+	source->type = type;
 	source->local = local;
 	source->path = xstrdup(path);
 }
diff --git a/odb/source.h b/odb/source.h
index e6698b73a3..45b72b81a0 100644
--- a/odb/source.h
+++ b/odb/source.h
@@ -1,6 +1,17 @@
 #ifndef ODB_SOURCE_H
 #define ODB_SOURCE_H
 
+enum odb_source_type {
+	/*
+	 * The "unknown" type, which should never be in use. This type mostly
+	 * exists to catch cases where the type field remains zeroed out.
+	 */
+	ODB_SOURCE_UNKNOWN,
+
+	/* The "files" backend that uses loose objects and packfiles. */
+	ODB_SOURCE_FILES,
+};
+
 /*
  * The source is the part of the object database that stores the actual
  * objects. It thus encapsulates the logic to read and write the specific
@@ -19,6 +30,9 @@ struct odb_source {
 	/* Object database that owns this object source. */
 	struct object_database *odb;
 
+	/* The type used by this source. */
+	enum odb_source_type type;
+
 	/*
 	 * Figure out whether this is the local source of the owning
 	 * repository, which would typically be its ".git/objects" directory.
@@ -58,6 +72,7 @@ struct odb_source *odb_source_new(struct object_database *odb,
  */
 void odb_source_init(struct odb_source *source,
 		     struct object_database *odb,
+		     enum odb_source_type type,
 		     const char *path,
 		     bool local);
 
diff --git a/odb/source-files.h b/odb/source-files.h
index 859a8f518a..803fa995fb 100644
--- a/odb/source-files.h
+++ b/odb/source-files.h
@@ -25,10 +25,13 @@ struct odb_source_files *odb_source_files_new(struct object_database *odb,
 void odb_source_files_free(struct odb_source_files *files);
 
 /*
  * Cast the given object database source to the files backend.
  */
 static inline struct odb_source_files *odb_source_files_downcast(struct odb_source *source)
 {
+	if (source->type != ODB_SOURCE_FILES)
+		BUG("trying to downcast source of type '%d' to files", source->type);
 	return container_of(source, struct odb_source_files, base);
 }
 
@@ -25,10 +25,13 @@ struct odb_source_files *odb_source_files_new(struct object_database *odb,
 
 /*
- * Cast the given object database source to the files backend.
+ * Cast the given object database source to the files backend. This will cause
+ * a BUG in case the source doesn't use this backend.
  */
 static inline struct odb_source_files *odb_source_files_downcast(struct odb_source *source)
-- 
2.53.0.797.g7842e34a66.dirty
Previous: Patrick SteinhardtNext: Patrick Steinhardt
Message 59 of 77 in “odb: make object database sources pluggable”
  1. 00/17 odb: make object database sources pluggablePatrick Steinhardt, Feb 23, 2026
  2. 01/17 odb: split `struct odb_source` into separate headerPatrick Steinhardt, Feb 23, 2026
  3. 02/17 odb: introduce "files" sourcePatrick Steinhardt, Feb 23, 2026
  4. 03/17 odb: embed base source in the "files" backendPatrick Steinhardt, Feb 23, 2026
  5. 04/17 odb: move reparenting logic into respective subsystemsPatrick Steinhardt, Feb 23, 2026
  6. 05/17 odb/source: introduce source type for robustnessPatrick Steinhardt, Feb 23, 2026
  7. 06/17 odb/source: make `free()` function pluggablePatrick Steinhardt, Feb 23, 2026
  8. 07/17 odb/source: make `reprepare()` function pluggablePatrick Steinhardt, Feb 23, 2026
  9. 08/17 odb/source: make `close()` function pluggablePatrick Steinhardt, Feb 23, 2026
  10. 09/17 odb/source: make `read_object_info()` function pluggablePatrick Steinhardt, Feb 23, 2026
  11. 10/17 odb/source: make `read_object_stream()` function pluggablePatrick Steinhardt, Feb 23, 2026
  12. 11/17 odb/source: make `for_each_object()` function pluggablePatrick Steinhardt, Feb 23, 2026
  13. 12/17 odb/source: make `freshen_object()` function pluggablePatrick Steinhardt, Feb 23, 2026
  14. 13/17 odb/source: make `write_object()` function pluggablePatrick Steinhardt, Feb 23, 2026
  15. 14/17 odb/source: make `write_object_stream()` function pluggablePatrick Steinhardt, Feb 23, 2026
  16. 15/17 odb/source: make `read_alternates()` function pluggablePatrick Steinhardt, Feb 23, 2026
  17. 16/17 odb/source: make `write_alternate()` function pluggablePatrick Steinhardt, Feb 23, 2026
  18. 17/17 odb/source: make `begin_transaction()` function pluggablePatrick Steinhardt, Feb 23, 2026
  19. Patrick SteinhardtFeb 23, 2026
  20. Junio C HamanoFeb 23, 2026
  21. Patrick SteinhardtFeb 24, 2026
  22. Justin ToblerMar 4, 2026
  23. Justin ToblerMar 4, 2026
  24. Justin ToblerMar 4, 2026
  25. Justin ToblerMar 4, 2026
  26. Justin ToblerMar 4, 2026
  27. Justin ToblerMar 4, 2026
  28. Justin ToblerMar 4, 2026
  29. Justin ToblerMar 4, 2026
  30. Justin ToblerMar 4, 2026
  31. Justin ToblerMar 4, 2026
  32. Justin ToblerMar 4, 2026
  33. Karthik NayakMar 5, 2026
  34. Karthik NayakMar 5, 2026
  35. Karthik NayakMar 5, 2026
  36. Karthik NayakMar 5, 2026
  37. Karthik NayakMar 5, 2026
  38. Karthik NayakMar 5, 2026
  39. Karthik NayakMar 5, 2026
  40. Patrick SteinhardtMar 5, 2026
  41. Karthik NayakMar 5, 2026
  42. Patrick SteinhardtMar 5, 2026
  43. Patrick SteinhardtMar 5, 2026
  44. Patrick SteinhardtMar 5, 2026
  45. Patrick SteinhardtMar 5, 2026
  46. Patrick SteinhardtMar 5, 2026
  47. Patrick SteinhardtMar 5, 2026
  48. Patrick SteinhardtMar 5, 2026
  49. Patrick SteinhardtMar 5, 2026
  50. Patrick SteinhardtMar 5, 2026
  51. Patrick SteinhardtMar 5, 2026
  52. Patrick SteinhardtMar 5, 2026
  53. Patrick SteinhardtMar 5, 2026
  54. 00/17 odb: make object database sources pluggablePatrick Steinhardt, Mar 5, 2026
  55. 01/17 odb: split `struct odb_source` into separate headerPatrick Steinhardt, Mar 5, 2026
  56. 02/17 odb: introduce "files" sourcePatrick Steinhardt, Mar 5, 2026
  57. 03/17 odb: embed base source in the "files" backendPatrick Steinhardt, Mar 5, 2026
  58. 04/17 odb: move reparenting logic into respective subsystemsPatrick Steinhardt, Mar 5, 2026
  59. 05/17 odb/source: introduce source type for robustnessPatrick Steinhardt, Mar 5, 2026
  60. 06/17 odb/source: make `free()` function pluggablePatrick Steinhardt, Mar 5, 2026
  61. 07/17 odb/source: make `reprepare()` function pluggablePatrick Steinhardt, Mar 5, 2026
  62. 08/17 odb/source: make `close()` function pluggablePatrick Steinhardt, Mar 5, 2026
  63. 09/17 odb/source: make `read_object_info()` function pluggablePatrick Steinhardt, Mar 5, 2026
  64. 10/17 odb/source: make `read_object_stream()` function pluggablePatrick Steinhardt, Mar 5, 2026
  65. 11/17 odb/source: make `for_each_object()` function pluggablePatrick Steinhardt, Mar 5, 2026
  66. 12/17 odb/source: make `freshen_object()` function pluggablePatrick Steinhardt, Mar 5, 2026
  67. 13/17 odb/source: make `write_object()` function pluggablePatrick Steinhardt, Mar 5, 2026
  68. 14/17 odb/source: make `write_object_stream()` function pluggablePatrick Steinhardt, Mar 5, 2026
  69. 15/17 odb/source: make `read_alternates()` function pluggablePatrick Steinhardt, Mar 5, 2026
  70. 16/17 odb/source: make `write_alternate()` function pluggablePatrick Steinhardt, Mar 5, 2026
  71. 17/17 odb/source: make `begin_transaction()` function pluggablePatrick Steinhardt, Mar 5, 2026
  72. Justin ToblerMar 5, 2026
  73. Justin ToblerMar 5, 2026
  74. Justin ToblerMar 5, 2026
  75. Justin ToblerMar 5, 2026
  76. Junio C HamanoMar 5, 2026
  77. Patrick SteinhardtMar 10, 2026

Read the whole thread, see it on lore, or plain text.

$ cat FOOTERMessages come from the public archive at lore.kernel.org/git, fetched every hour. The front page is chosen and written each morning by an AI editor and can be wrong; the threads themselves are the record. About and API. For agents: an MCP server at https://gitlist.dev/mcp, and any thread, story or person page as Markdown by adding .md to its URL (or sending Accept: text/markdown). Details in /llms.txt.